/*
    Copyright 2010 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#else
    #define stricmp strcasecmp
    #define stristr strcasestr
#endif

typedef unsigned char   u8;



#define VER     "0.1"



void MHDecrypt(u8 *msg, u8 *pwd);
int unbase64(u8 *in, int insz, u8 *out, int outsz);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u8      buff[4096],
            *p;

    setbuf(stdout, NULL);

    fputs("\n"
        "Phoenix mail password decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <account.ini/password>\n"
            "\n", argv[0]);
        goto quit;
    }

    fd = fopen(argv[1], "rb");
    if(fd) {
        while(fgets(buff, sizeof(buff), fd)) {
            for(p = buff; *p && (*p != '\n') && (*p != '\r'); p++);
            *p = 0;
            p = strchr(buff, '=');
            if(!p) continue;
            *p++ = 0;
            if(!stricmp(buff, "password")) {
                MHDecrypt(buff, p);
            } else if(!stricmp(buff, "user")) {
                printf("- %s: %s\n", buff, p);
            }
        }
        fclose(fd);
    } else {
        MHDecrypt("password", argv[1]);
    }

quit:
    printf("\nPress RETURN to quit\n");
    fgetc(stdin);
    return(0);
}



void MHDecrypt(u8 *msg, u8 *pwd) {
    int     i,
            len;
    u8      *p;

    p = strdup(pwd);
    len = strlen(pwd) - 1;  // keep the last char out
    for(i = 0; i < len; i++) {
        p[i] = pwd[(len - 1) - i];
    }

    len = strlen(p);
    len = unbase64(p, len, p, len);

    len = ((p[0] << 4) | (p[1] >> 4)) & 0xff;
    for(i = 0; i < len; i++) {
        pwd[i] = (p[i+1] << 4) | (p[i+2] >> 4);
    }
    pwd[i] = 0;

    printf("- %s: %s\n", msg, pwd);
}



int unbase64(u8 *in, int insz, u8 *out, int outsz) {
    int     xlen,
            a   = 0,
            b   = 0,
            c,
            step;
    u8      *limit,
            *data,
            *p;
    static const u8 base[128] = {   // supports also the Gamespy base64 and URLs
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0x00,0x3e,0x00,0x3f,
        0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,
        0x0f,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x3e,0x00,0x3f,0x00,0x3f,
        0x00,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,
        0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,0x00,0x00,0x00,0x00,0x00
    };

    if(insz < 0) insz = strlen(in);
    xlen = ((insz >> 2) * 3) + 1;    // NULL included in output for text
    if(outsz < xlen) return(-1);
    data = in;

    p = out;
    limit = data + insz;

    for(step = 0; /* data < limit */; step++) {
        do {
            if(data >= limit) {
                c = 0;
                break;
            }
            c = *data;
            data++;
            if((c == '=') || (c == '_')) {  // supports also the Gamespy base64
                c = 0;
                break;
            }
        } while(c && ((c <= ' ') || (c > 0x7f)));
        if(!c) break;

        switch(step & 3) {
            case 0: {
                a = base[c];
                break;
            }
            case 1: {
                b = base[c];
                *p++ = (a << 2)        | (b >> 4);
                break;
            }
            case 2: {
                a = base[c];
                *p++ = ((b & 15) << 4) | (a >> 2);
                break;
            }
            case 3: {
                *p++ = ((a & 3) << 6)  | base[c];
                break;
            }
        }
    }
    *p = 0;
    return(p - out);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}


